
# Simple Service Discovery Protocol (SSDP)

Curated from:
[https://en.wikipedia.org/wiki/Simple_Service_Discovery_Protocol](https://en.wikipedia.org/wiki/Simple_Service_Discovery_Protocol)
[https://williamboles.me/discovering-whats-out-there-with-ssdp/](https://williamboles.me/discovering-whats-out-there-with-ssdp/)
[https://coolaj86.com/articles/adventures-in-upnp-on-osx/](https://coolaj86.com/articles/adventures-in-upnp-on-osx/)

Simple Service Discovery Protocol (SSDP) is a network protocol based on the Internet protocol suite for advertisement and discovery of network services and presence information.

SSDP is the basis of the discovery protocol of Universal Plug and Play (UPnP).

SSDP is a text-based protocol based on HTTPU. It uses UDP as the underlying transport protocol. Services are announced by the hosting system with multicast addressing to a specifically designated IP multicast address at UDP port number 1900. In IPv4, the multicast address is 239.255.255.250 and SSDP over IPv6 uses the address set ff0X::c for all scope ranges indicated by X.

This results in the following well-known practical multicast addresses for SSDP:

- 239.255.255.250 (IPv4 site-local address)
- [FF02::C] (IPv6 link-local)
- [FF05::C] (IPv6 site-local)
- [FF08::C] (IPv6 organization-local)
- [FF0E::C] (IPv6 global)

Device search requests and advertisements are supported by running HTTP on top of UDP (port 1900) using multicast (known as HTTPMU). Responses to search requests are also sent over UDP, but are instead sent using unicast (known as HTTPU).

For multicast messages, IANA has reserved the IPv4 address 239.255.255.250 and port 1900 for SSDP.

SSDP uses the HTTP method NOTIFY to announce the establishment or withdrawal of services (presence) information to the multicast group. A client that wishes to discover available services on a network, uses method M-SEARCH. Responses to such search requests are sent via unicast addressing to the originating address and port number of the multicast request.

SSDP messages conform to the header field format of HTTP 1.1. It's important to note that SSDP does not allow any message to contain a body; everything is shared via those header fields.

An SSDP node is either a root device or control point. A root device offers one or more SSDP services; a control point uses SSDP services.

When a root device is responding to a discovery message it does so by sending a unicast (to a single specific node) message directly to the control point.

Example search request:

Important!: Each line must end in `\r\n`

```
M-SEARCH * HTTP/1.1
HOST: 239.255.255.250:1900
MAN: "ssdp:discover"
MX: 1
ST: urn:schemas-upnp-org:device:InternetGatewayDevice:1
USER-AGENT: DAVE
```

Where:
- HOST: The host and port the message will be sent to
- MAN: message type (always "ssdp:discover" for an M-SEARCH query)
- ST: Search Target
- MX: The maximum wait response time (MX) in seconds that a root device can take before responding. The MX field is an attempt to overcome a scaling issue implicit with SSDP. SSDP is a chatty protocol, in a network with a significant number of nodes that host SSDP services, sending an M-SEARCH message could result in accidentally DDOS-ing the questing node due to too many services responding at once. The MX field instructs the root device to wait a random time between 0 and MX before attempting to respond - this should allow the responses to be spaced out enough to ease the processing strain on the control point. The MX value should be between 1 and 5. Even with the MX workaround, SSDP is recommended only to be used in home or small office networks.

Example Response:

``` 
HTTP/1.1 200 OK
CACHE-CONTROL: max-age=3600
ST: urn:schemas-upnp-org:device:InternetGatewayDevice:1
USN: uuid:0175c106-5400-10f8-802d-b0a7374360b7::urn:schemas-upnp-org:device:InternetGatewayDevice:1
EXT:
SERVER: Roku UPnP/1.0 MiniUPnPd/1.4
LOCATION: http://192.168.1.104:8060/dial/dd.xml
```

Where:
- CACHE-CONTROL: The cache-control (CACHE-CONTROL) value to determine for how long the message is valid.
- USN: The unique service name (USN) to identify the service.
- SERVER: The server system information (SERVER) value providing information in the following format: [OS-Name] UPnP/[Version] [Product-Name]/[Product-Version].
- LOCATION: The location URL (LOCATION) to allow the control point to gain more information about this service.

The EXT field is required for backwards compatibility with UPnP 1.0 but can otherwise be ignored.

---

ADVERTISEMENT

Alive, update and byebye message examples:

![Alive](examples/notify/alive.txt)
![Update](examples/notify/update.txt)
![Byebye](examples/notify/byebye.txt)

Where:

The notification type (NT) that defines the service it offers (the equivalent of ST in an M-Search message).
The notification subtype (NTS), for an alive message this will always be ssdp:alive (the equivalent of MAN in an M-Search message).

A byebye message has a very similar structure to both an alive and update message only omitting the LOCATION header field and having a different NTS value.

Now that we have an understanding of what SSDP is let's get back to the solution.

---

## Misc

Interesting articles on SSDP:

- [https://blog.cloudflare.com/ssdp-100gbps/](https://blog.cloudflare.com/ssdp-100gbps/)

