
# mDNS (Multicast DNS)

Tl;dr a protocol allowing for the mapping of ip addresses to domain names. Consequential as it can work with DNS-SD to allow for the discovery of services on a local network with zeroconf.

The multicast aspect here is important as it allows for the broadcasting and listening of these records across all computers on a LAN.

Using the dns-sd cli tool we can introspect the services registered on a network to discover their locations etc.

E.g:

```
dns-sd  -B  _services._dns-sd._udp  local.
```

May show:

```
Browsing for _services._dns-sd._udp.local.
DATE: ---Tue 29 Dec 2020---
12:22:50.964  ...STARTING...
Timestamp     A/R    Flags  if Domain               Service Type         Instance Name
12:22:50.965  Add        3   6 .                    _tcp.local.          _airplay
12:22:50.965  Add        3   6 .                    _tcp.local.          _googlecast
12:22:50.965  Add        3   6 .                    _tcp.local.          _hue
12:22:50.965  Add        2   6 .                    _tcp.local.          _hap
```

Querying for the specific service type reveals more info e.g:

```
dns-sd -B _hap._tcp. local
```

Shows:

```
Browsing for _hap._tcp..local
DATE: ---Tue 29 Dec 2020---
12:33:10.378  ...STARTING...
Timestamp     A/R    Flags  if Domain               Service Type         Instance Name
12:33:10.379  Add        2   6 local.               _hap._tcp.           Philips hue - 011828
```

Querying by the name and service:

```
dns-sd -L "Philips Hue - 011828" _hap._tcp. local
```

Shows:

```
Lookup Philips Hue - 011828._hap._tcp..local
DATE: ---Tue 29 Dec 2020---
12:35:11.595  ...STARTING...
12:35:11.939  Philips\032hue\032-\032011828._hap._tcp.local. can be reached at Philips-hue.local.:8080 (interface 6)
 c#=41 ff=1 id=CB:54:C7:27:66:B5 md=BSB002 pv=1.1 sf=1 s#=3 ci=2
```

Navigating to `Philips-hue.local.:8080` shows service information

Alternatively, the whole listing can be shown using the Zone format as per:

```
dns-sd -Z _hap._tcp. local
```

Shows:

```
Browsing for _hap._tcp..local
DATE: ---Tue 29 Dec 2020---
12:37:03.153  ...STARTING...

; To direct clients to browse a different domain, substitute that domain in place of '@'
lb._dns-sd._udp                                 PTR     @

; In the list of services below, the SRV records will typically reference dot-local Multicast DNS names.
; When transferring this zone file data to your unicast DNS server, you'll need to replace those dot-local
; names with the correct fully-qualified (unicast) domain name of the target host offering the service.

_hap._tcp                                       PTR     Philips\032hue\032-\032011828._hap._tcp
Philips\032hue\032-\032011828._hap._tcp         SRV     0 0 8080 Philips-hue.local. ; Replace with unicast FQDN of target host
Philips\032hue\032-\032011828._hap._tcp         TXT     "c#=41" "ff=1" "id=CB:54:C7:27:66:B5" "md=BSB002" "pv=1.1" "sf=1" "s#=3" "ci=2"
```
