
# UPnP 101

Tl;dr - Combination of Protocols to facilitate communication between devices. Open and managed by the Open Connectivity Foundation (OCF)

Mostly taken from: [UPnP Wiki](https://en.wikipedia.org/wiki/Universal_Plug_and_Play).

Universal Plug and Play (UPnP) is a set of networking protocols that permits networked devices, such as personal computers, printers, Internet gateways, Wi-Fi access points and mobile devices to seamlessly discover each other's presence on the network and establish functional network services for data sharing, communications, and entertainment. 

UPnP assumes the network runs Internet Protocol (IP) and then leverages HTTP, on top of IP, in order to provide device/service description, actions, data transfer and eventing. Device search requests and advertisements are supported by running HTTP on top of UDP (port 1900) using multicast (known as HTTPMU). Responses to search requests are also sent over UDP, but are instead sent using unicast (known as HTTPU).

It is a distributed, open architecture protocol based on established standards such as the Internet Protocol Suite (TCP/IP), HTTP, XML, and SOAP.

UPnP control points (CPs) are devices which use UPnP protocols to control UPnP controlled devices (CDs).

## Protocol (Tl;dr)

Uses HTTP, XML and SOAP.

Uses UDP under the hood on outbound for search requests (multicast). Responses to search requests are UDP unicast.

### 1.) Addressing

Via IP addresses. Each device must implement a DHCP client and search for a DHCP server when the device is first connected to the network. If no DHCP server is available, the device must assign itself an address.

 The process by which a UPnP device assigns itself an address is known within the UPnP Device Architecture as AutoIP.
 
 If during the DHCP transaction, the device obtains a domain name, for example, through a DNS server or via DNS forwarding, the device should use that name in subsequent network operations; otherwise, the device should use its IP address.
 
### 2.) Discovery

Tl;dr - Uses SSDP for this part. SSDP has a mix of query and active "alive" messages that can be listened to.

Once a device has established an IP address, the next step in UPnP networking is discovery. 

The UPnP discovery protocol is known as the Simple Service Discovery Protocol (SSDP). When a device is added to the network, SSDP allows that device to advertise its services to control points on the network. This is achieved by sending SSDP alive messages. When a control point is added to the network, SSDP allows that control point to actively search for devices of interest on the network or listen passively to the SSDP alive messages of device. 

The fundamental exchange is a discovery message containing a few essential specifics about the device or one of its services, for example, its type, identifier, and a pointer (network location) to more detailed information.

### 3.) Description (Whats available)

Tl;dr - Discovery phase only associates the device, description contains more detailed info on capabilities and interactions etc.

After a control point has discovered a device, the control point still knows very little about the device. For the control point to learn more about the device and its capabilities, or to interact with the device, the control point must retrieve the device's description from the location (URL) provided by the device in the discovery message.

The UPnP Device Description is expressed in XML and includes vendor-specific manufacturer information like the model name and number, serial number, manufacturer name, (presentation) URLs to vendor-specific web sites, etc. 

The description also includes a list of any embedded "services". For each service, the Device Description document lists the URLs for control, eventing and service description. Each service description includes a list of the commands, or actions, to which the service responds, and parameters, or arguments, for each action; the description for a service also includes a list of variables; these variables model the state of the service at run time, and are described in terms of their data type, range, and event characteristics.

### 4.) Control (Interaction)

Having retrieved a description of the device, the control point can send actions to a device's service.

To do this, a control point sends a suitable control message to the control URL for the service (provided in the device description). Control messages are also expressed in XML using the Simple Object Access Protocol (SOAP). Much like function calls, the service returns any action-specific values in response to the control message. The effects of the action, if any, are modeled by changes in the variables that describe the run-time state of the service.

### Events

Tl;dr - there is a notion of events which can publish only state changes for listed variables.

Another capability of UPnP networking is event notification, or eventing. The event notification protocol defined in the UPnP Device Architecture is known as General Event Notification Architecture (GENA). A UPnP description for a service includes a list of actions the service responds to and a list of variables that model the state of the service at run time. The service publishes updates when these variables change, and a control point may subscribe to receive this information. The service publishes updates by sending event messages. Event messages contain the names of one or more state variables and the current value of those variables. These messages are also expressed in XML. A special initial event message is sent when a control point first subscribes; this event message contains the names and values for all evented variables and allows the subscriber to initialize its model of the state of the service. To support scenarios with multiple control points, eventing is designed to keep all control points equally informed about the effects of any action. Therefore, all subscribers are sent all event messages, subscribers receive event messages for all "evented" variables that have changed, and event messages are sent no matter why the state variable changed (either in response to a requested action or because the state the service is modeling changed).

### Presentation

The final step in UPnP networking is presentation. If a device has a URL for presentation, then the control point can retrieve a page from this URL, load the page into a web browser, and depending on the capabilities of the page, allow a user to control the device and/or view device status. The degree to which each of these can be accomplished depends on the specific capabilities of the presentation page and device.

## UPnP AV standards

UPnP AV architecture is an audio and video extension of the UPnP, supporting a variety of devices such as TVs, VCRs, CD/DVD players/jukeboxes, settop boxes, stereos systems, MP3 players, still image cameras, camcorders, electronic picture frames (EPFs), and personal computers. The UPnP AV architecture allows devices to support different types of formats for the entertainment content, including MPEG2, MPEG4, JPEG, MP3, Windows Media Audio (WMA), bitmaps (BMP), and NTSC, PAL or ATSC formats. Multiple types of transfer protocols are supported, including IEEE 1394, HTTP, RTP and TCP/IP.

### UPnP AV components

#### Media server

A UPnP AV media server is the UPnP-server ("master" device) that provides media library information and streams media-data (like audio/video/picture/files) to UPnP clients on the network. It is a computer system or a similar digital appliance that stores digital media, such as photographs, movies, or music and shares these with other devices.

UPnP AV media servers provide a service to UPnP AV client devices, so-called control points, for browsing the media content of the server and request the media server to deliver a file to the control point for playback.

UPnP media servers are available for most operating systems and many hardware platforms. UPnP AV media servers can either be categorized as software-based or hardware-based. Software-based UPnP AV media servers can be run on a PC. Hardware-based UPnP AV media servers may run on any NAS devices or any specific hardware for delivering media, such as a DVR.

UPnP MediaServer ControlPoint - which is the UPnP-client (a 'slave' device) that can auto-detect UPnP-servers on the network to browse and stream media/data-files from them.
UPnP MediaRenderer DCP - which is a 'slave' device that can render (play) content.
UPnP RenderingControl DCP - control MediaRenderer settings; volume, brightness, RGB, sharpness, and more.
UPnP Remote User Interface (RUI) client/server - which sends/receives control-commands between the UPnP-client and UPnP-server over network, (like record, schedule, play, pause, stop, etc.).

## NAT traversal
One solution for NAT traversal, called the Internet Gateway Device Protocol (IGD Protocol), is implemented via UPnP. Many routers and firewalls expose themselves as Internet Gateway Devices, allowing any local UPnP control point to perform a variety of actions, including retrieving the external IP address of the device, enumerating existing port mappings, and adding or removing port mappings. By adding a port mapping, a UPnP controller behind the IGD can enable traversal of the IGD from an external address to an internal client.

---

### Summary / Misc

- UPnP is a collection of protocols that facilitate easy connection and comms between devices (on the same, IP compatible, network)

- There exist the following logical components / concepts:
    - a Control Point (CP); that make use of UPnP protocols to control other UPnP compatible devices
    - A Controlled Device (CD); that can be manipulated via its services via other CPs
    - DCP - Device control protocol
    - UPnP devices may choose to surface any amount of information on its description (such as presentation urls)
    - Functional aspects of behaviour and data and vended via the use of "services"
    - There is such a thing as a "Basic Device" which implements all rules for discovery and description but offers no services

- UPnP certified devices (and those eligible for logos) must contain at least 1 service
- However, if other UPnP Standardized Device types are embedded within a Basis Device (for example, a MediaServer or MediaRenderer), then those Standardized Device types are eligible to carry the UPnP logo.
- There is a notion of root device and embedded device types. I.e a TV could have a root basic Device Type but Light controls within. 

- There is the notion of "multi-homed" devices which have more than one network interface that is configured to register via UPnP.

Useful links / specs:

- [http://upnp.org/specs/basic/UPnP-basic-Basic-v1-Device.pdf](http://upnp.org/specs/basic/UPnP-basic-Basic-v1-Device.pdf)
- [https://openconnectivity.org/upnp-specs/UPnP-arch-DeviceArchitecture-v2.0-20200417.pdf](https://openconnectivity.org/upnp-specs/UPnP-arch-DeviceArchitecture-v2.0-20200417.pdf). 
- [https://medium.com/@djboris/digging-into-upnp-by-searching-a-sonos-api-5e10e080a232](https://medium.com/@djboris/digging-into-upnp-by-searching-a-sonos-api-5e10e080a232)
- [https://www.researchgate.net/publication/220791510_Online_UPnP_AV_Device_Database_for_Quick_and_Easy_Capability_Checking#pf5](https://www.researchgate.net/publication/220791510_Online_UPnP_AV_Device_Database_for_Quick_and_Easy_Capability_Checking#pf5) 

#### Acronyms

ARP Address Resolution Protocol 
CP Control Point
DCP Device Control Protocol 
DDD Device Description Document 
DHCP Dynamic Host Configuration Protocol 
DNS Domain Name System 
GENA General Event Notification Architecture 
HTML Hypertext Markup Language 
HTTP Hypertext Trrdansfer Protocol 
SCPD Service Control Protocol Description
SOAP Simple Object Access Protocol
SSDP Simple Service Discovery Protocol
UDA UPnP Device Architecture
UPC Universal Product Code
URI Uniform Resource Identifier
URL Uniform Resource Locator
URN Uniform Resource Name
UUID Universally Unique Identifier
XML Extensible Markup Language
