
# Samsung TV (Q90R)


## UPnP

Appears to have UPnP enabled by default, example device types found:

- [urn:dial-multiscreen-org:device:dialreceiver:1] http://192.168.1.159:7678/nservice/
- [urn:samsung.com:device:ScreenSharing:1] http://192.168.1.159:9119/screen_sharing
- [urn:schemas-upnp-org:device:MediaRenderer:1]  http://192.168.1.159:9197/dmr


---


Related articles:

- 
- [https://community.hubitat.com/t/samsung-tv-integration-question-driver-info-starting-post-127/8341/190](https://community.hubitat.com/t/samsung-tv-integration-question-driver-info-starting-post-127/8341/190)

## Setting up device to allow SetVolume,SetMute commands etc.

This may be a common UPnP flow, but certain actions like SetVolume and SetMute are not executable (return 501) if the target device is not in a certain state. Identical requests previously returning 501s subsequently return 200 when using another UPnP app that follows some workflow on the same device - this strongly suggests either device state is being manipulated on the media server side prior and post other set actions OR some IP / MAC address is stored on registration.

Current working theories:

- AVTransport instance id needs to be allocated or set to a certain state before "set" actions can be used.
- A subscription to renderer results in the media device becoming aware of the CP.
- A upnp device alive command needs to be sent from the originating control point prior to invocation

This needs to be replicated consistently...

More related info here:

- [http://upnp.org/specs/av/UPnP-av-MediaRenderer-v1-Device.pdf](http://upnp.org/specs/av/UPnP-av-MediaRenderer-v1-Device.pdf)

*** UPnP AV profile could be worth digging into.


## Dial (Discovery and Launch Protocol)

DIAL spec:

[https://docs.google.com/viewer?a=v&pid=sites&srcid=ZGlhbC1tdWx0aXNjcmVlbi5vcmd8ZGlhbHxneDoyNzlmNzY3YWJlMmY1MjZl](https://docs.google.com/viewer?a=v&pid=sites&srcid=ZGlhbC1tdWx0aXNjcmVlbi5vcmd8ZGlhbHxneDoyNzlmNzY3YWJlMmY1MjZl)

Two main components:

- DIAL Service Discovery
- DIAL REST Service.

Process:

1.) MSEARCH

2.) Hit LOCATION URL -> check "Application-URL: http://192.168.1.159:8080/ws/app/" header value.


### DIAL REST Service

```
http://192.168.1.159:8080/ws/app/
```

E.g Check if DIAL Application Installed:

```
GET http://192.168.1.159:8080/ws/app/Netflix
```

Open DIAL application:

``` 
POST http://192.168.1.159:8080/ws/apps/Netflix
```

### DIAL Registry

All names of registered applications are held here:

[http://www.dial-multiscreen.org/dial-registry/namespace-database](http://www.dial-multiscreen.org/dial-registry/namespace-database)



### WOL

Also seems to support WOL based on SSDP events i.e:

```
CACHE-CONTROL: max-age=1800
DATE: Thu, 31 Dec 2020 17:43:39 GMT
EXT:
LOCATION: http://192.168.1.159:7678/nservice/
SERVER: Samsung-Linux/4.1, UPnP/1.0, Samsung_UPnP_SDK/1.0
ST: urn:dial-multiscreen-org:service:dial:1
USN: uuid:558ee8d1-72bf-4c36-8b64-55554f57f454::urn:dial-multiscreen-org:service:dial:1
WAKEUP: MAC=8c:79:f5:7e:0b:61;Timeout=10
Content-Length: 0
BOOTID.UPNP.ORG: 5
```


### Dial.XML / RCR 

Related Remote control links:

- [https://github.com/Ape/samsungctl](https://github.com/Ape/samsungctl)
- [https://github.com/svilborg/samsungtv](https://github.com/svilborg/samsungtv)
- [http://sc0ty.pl/2012/02/samsung-tv-network-remote-control-protocol/](http://sc0ty.pl/2012/02/samsung-tv-network-remote-control-protocol/)
- [https://gist.github.com/danielfaust/998441](https://gist.github.com/danielfaust/998441)
- [https://github.com/Bntdumas/SamsungIPRemote](https://github.com/Bntdumas/SamsungIPRemote)
- [https://github.com/kyleaa/homebridge-samsungtv2016](https://github.com/kyleaa/homebridge-samsungtv2016)

## Samsung TV Remote Control Receiver (Non UPnP / DLNA  / DIAL? - Samsung Specific?)

```
http://192.168.1.159:8001/api/v2/ 
```


## Samsung Smart Remote


## Websocket comms? Same as Samsung Smart Remote?

Requires a token. Figure out how to obtain one...


Steps:

1.) Ensure cert is accepted or accept self signed certs etc

2.) Make initial request to secure WS location

```
wscat -n --connect wss://192.168.1.159:8002/api/v2/channels/samsung.remote.control
```

Example valid response:

``` 
{
  "data": {
    "clients": [
      {
        "attributes": {
          "name": null
        },
        "connectTime": 1609731033281,
        "deviceName": "Smart Device",
        "id": "5a4b41d-9465-44f4-91c5-dab4d033ec73",
        "isHost": false
      }
    ],
    "id": "5a4b41d-9465-44f4-91c5-dab4d033ec73",
    "token": "55325339"
  },
  "event": "ms.channel.connect",
```

Example error response:

```
{"data":{"message":"can't parse socket message!"},"event":"ms.error"}
```

Example command request:

```
{
  "method": "ms.remote.control",
  "params": {
    "Cmd": "Click",
    "DataOfCmd": "KEY_VOLUP",
    "Option": "false",
    "TypeOfRemote": "SendRemoteKey"
  }
}
```


--- Misc:

ms.remote.control


Remote:

```
"wss://${deviceIp}:8002/api/v2/channels/samsung.remote.control?name=${name}&token=${token}" 
```

Application:
``` 
"ws://${deviceIp}:8001/api/v2/applications?name=${name}"
```




